@@ -69,8 +69,8 @@ static inline int fio_set_odirect(int fd)
 /*
  * pset binding hooks for fio
  */
-#define fio_setaffinity(td)		\
-	pset_bind((td)->o.cpumask, P_PID, (td)->pid, NULL)
+#define fio_setaffinity(pid, cpumask)		\
+	pset_bind(&(cpumask), P_PID, (pid), NULL)
 #define fio_getaffinity(pid, ptr)	({ 0; })
 
 #define fio_cpu_clear(mask, cpu)	pset_assign(PS_NONE, (cpu), NULL)
